/**
 * Created by xieli on 2017/2/23.
 */

module cc {

    export class Rect {
        /** !#en
         Constructor of cc.Rect class.
         see {{#crossLink "cc/rect:method"}} cc.rect {{/crossLink}} for convenience method.
         !#zh
         cc.Rect类的构造函数。可以通过 {{#crossLink "cc/rect:method"}} cc.rect {{/crossLink}} 简便方法进行创建。
         */
        constructor(x?: number, y?: number, w?: number, h?: number) {
            this.x = x || 0;
            this.y = y || 0;
            this.width = w || 0;
            this.height = h || 0;
        }

        x: number;
        y: number;
        width: number;
        height: number;

//     constructor(x?: number, y?: number, w?: number, h?: number):  {
//
// }
        /** !#en TODO
         !#zh 矩形 x 轴上的最小值。 */
        // xMin: number;
        // /** !#en TODO
        //  !#zh 矩形 y 轴上的最小值。 */
        // yMin: number;
        // /** !#en TODO
        //  !#zh 矩形 x 轴上的最大值。 */
        // xMax: number;
        // /** !#en TODO
        //  !#zh 矩形 y 轴上的最大值。 */
        // yMax: number;
        // /** !#en TODO
        //  !#zh 矩形的中心点。 */
        // center: number;
        /** !#en TODO
         !#zh 矩形的大小。 */

        /** !#en Creates a rectangle from two coordinate values.
         !#zh 根据指定 2 个坐标创建出一个矩形区域。

         @example
         ```js
         cc.Rect.fromMinMax(cc.v2(10, 10), cc.v2(20, 20)); // Rect {x: 10, y: 10, width: 10, height: 10};
         ```
         */
// fromMinMax(v1
// :
// cc.Vec2, v2
// :
// cc.Vec2
// ):
// Rect;

        /** !#en Checks if rect contains.
         !#zh
         判断 2 个矩形是否有包含。<br/>
         返回 1 为 a 包含 b，如果 -1 为 b 包含 a,
         0 这则都不包含。
         @param a Rect a
         @param b Rect b

         @example
         ```js
         var a = new cc.rect(0, 0, 10, 10);
         var b = new cc.rect(5, 5, 5, 5);
         var c = new cc.rect(20, 20, 10, 10);
         cc.Rect.contain(a, b); //  1;
         cc.Rect.contain(b, a); // -1;
         cc.Rect.contain(a, c); //  0;
         ```
         */
// contain(a
// :
// (x: number, y: number, w: number, h: number) => void, b
// :
// (x: number, y: number, w: number, h: number) => void
// ):
// number;

        /** !#en TODO
         !#zh 克隆一个新的 Rect。

         @example
         ```js
         var a = new cc.rect(0, 0, 10, 10);
         a.clone();// Rect {x: 0, y: 0, width: 10, height: 10}
         ```
         */
// clone()
// :
// Rect;

        /** !#en TODO
         !#zh 是否等于指定的矩形。
         @param other !#en
         Constructor of cc.Rect class.
         see {{#crossLink "cc/rect:method"}} cc.rect {{/crossLink}} for convenience method.
         !#zh
         cc.Rect类的构造函数。可以通过 {{#crossLink "cc/rect:method"}} cc.rect {{/crossLink}} 简便方法进行创建。

         @example
         ```js
         var a = new cc.rect(0, 0, 10, 10);
         var b = new cc.rect(0, 0, 10, 10);
         a.equals(b);// true;
         ```
         */
// equals(other
// :
// (x: number, y: number, w: number, h: number) => void
// ):
// boolean;

        /** !#en TODO
         !#zh 线性插值
         @param to !#en
         Constructor of cc.Rect class.
         see {{#crossLink "cc/rect:method"}} cc.rect {{/crossLink}} for convenience method.
         !#zh
         cc.Rect类的构造函数。可以通过 {{#crossLink "cc/rect:method"}} cc.rect {{/crossLink}} 简便方法进行创建。
         @param ratio the interpolation coefficient.
         @param out optional, the receiving vector.

         @example
         ```js
         var a = new cc.rect(0, 0, 10, 10);
         var b = new cc.rect(50, 50, 100, 100);
         update (dt) {
		   // method 1;
		   var c = a.lerp(b, dt * 0.1);
		   // method 2;
		   a.lerp(b, dt * 0.1, c);
		}
         ```
         */
// lerp(to
// :
// (x: number, y: number, w: number, h: number) => void, ratio
// :
// number, out
// :
// (x: number, y: number, w: number, h: number) => void
// ):
// Rect;

        /** !#en TODO
         !#zh 转换为方便阅读的字符串

         @example
         ```js
         var a = new cc.rect(0, 0, 10, 10);
         a.toString();// "(0.00, 0.00, 10.00, 10.00)";
         ```
         */
// toString()
// :
// string;


        /** !#en TODO
         !#zh 当前矩形与指定矩形是否相交。
         @param rect !#en
         Constructor of cc.Rect class.
         see {{#crossLink "cc/rect:method"}} cc.rect {{/crossLink}} for convenience method.
         !#zh
         cc.Rect类的构造函数。可以通过 {{#crossLink "cc/rect:method"}} cc.rect {{/crossLink}} 简便方法进行创建。

         @example
         ```js
         var a = new cc.rect(0, 0, 10, 10);
         var b = new cc.rect(0, 0, 20, 20);
         a.intersects(b);// true
         ```
         */
// intersects(rect
// :
// (x: number, y: number, w: number, h: number) => void
// ):
        void;

        /** !#en TODO
         !#zh 当前矩形是否包含指定坐标点。
         Returns true if the point inside this rectangle.

         @example
         ```js
         var a = new cc.rect(0, 0, 10, 10);
         var b = new cc.v2(0, 5);
         a.contains(b);// true
         ```
         */
// contains(point
// :
// cc.Vec2
// ):
// void;

        /** !#en Returns true if the other rect totally inside this rectangle.
         !#zh 当前矩形是否包含指定矩形。
         @param rect !#en
         Constructor of cc.Rect class.
         see {{#crossLink "cc/rect:method"}} cc.rect {{/crossLink}} for convenience method.
         !#zh
         cc.Rect类的构造函数。可以通过 {{#crossLink "cc/rect:method"}} cc.rect {{/crossLink}} 简便方法进行创建。

         @example
         ```js
         var a = new cc.rect(0, 0, 10, 10);
         var b = new cc.rect(0, 0, 20, 20);
         a.containsRect(b);// true
         ```
         */
// containsRect(rect
// :
// (x: number, y: number, w: number, h: number) => void
// ):
// void;
    }
}