module game {

    export class Map <K, V> {
        m_keys: Array<K> = null;
        _data: Object = null;
        size: number = 0;

        constructor() {
            this.m_keys = new Array();
            this._data = new Object();
        }


        forEach(callbackfn: (value: V, index: K, map: Map<K, V>) => void, thisArg: any): void {
            if (typeof callbackfn != 'function') {
                return;
            }
            if (thisArg == null) {
                return;
            }

            let len = this.m_keys.length;
            for (let i = 0; i < len; i++) {
                let k: any = this.m_keys[i];
                callbackfn.call(thisArg, this._data[k], k, this);
            }
        }


        delete(key: K): boolean {
            for (let i = 0; i < this.m_keys.length; i++) {
                if (key === this.m_keys[i]) {
                    let del_keys = this.m_keys.splice(i, 1);
                    for (let k in del_keys) {
                        let data: any = del_keys[k];
                        this._data[data] = null;
                    }
                    this.size = this.m_keys.length;
                    return true;
                }
            }

            return false;
        }


        getKey(index) {
            if (index == null) {
                return null;
            }
            return this.m_keys[index];
        }


        getByIndex(index: number): V {
            for (let i = 0; i < this.m_keys.length; i++) {
                if (index ===  i) {
                    let keybuff: any = this.m_keys[i];
                    return this._data[keybuff];
                }
            }
            return null;
        }
        // /**
        //  * 获取某键对应的值
        //  * @param {String} key
        //  * @return {Object} value
        //  */
        get(key: K): V {
            for (let i = 0; i < this.m_keys.length; i++) {
                if (key === this.m_keys[i]) {
                    let keybuff: any = key;
                    return this._data[keybuff];
                }
            }
            return null;
        }

        has(key: K): boolean {
            let ret = this.get(key);
            if (ret) {
                return true;
            }
            return false;
        }

        set(key: K, value: V) {
            this.delete(key); // 替换掉旧的数据
            this.m_keys.push(key);
            let keybuff: any = key;
            this._data[keybuff] = value;
            this.sort();
            this.size = this.m_keys.length;
            return true;
        }


        //排序
        sort() {
            // this.m_keys.sort( function (a:K, b:K) {
            //     return b < a;
            // });
        }


        //删除所有数据
        clear() {
            this.m_keys = null;
            this._data = null;
            this.m_keys = new Array();
            this._data = new Object();
            this.size = 0;
        }


        /**
         * 判断Map是否为空
         */
        isEmpty() {
            return this.m_keys.length == 0;
        }


    }


    export class ObjMap <  V> {
        m_keys: Array<any> = null;
        _data: Object = null;
        size: number = 0;

        constructor(obj: Object = null) {
            this.m_keys = new Array();
            this._data = new Object();

            if (obj) {
                for (let p in obj) {
                    let data = obj[p];
                    this.set(p, data);
                }
            }
        }

        getObject(): Object {
            let obj: Object = new Object();
            this.forEach(( v,k ) => {
                obj[k] = v;
            }, this)
            return obj;
        }


        forEach(callbackfn: (value: V, index: any, map: ObjMap< V>) => void, thisArg: any): void {
            if (typeof callbackfn != 'function') {
                return;
            }
            if (thisArg == null) {
                return;
            }

            let len = this.m_keys.length;
            for (let i = 0; i < len; i++) {
                let k: any = this.m_keys[i];
                callbackfn.call(thisArg, this._data[k], k, this);
            }
        }


        delete(key: any): boolean {
            for (let i = 0; i < this.m_keys.length; i++) {
                if (key === this.m_keys[i]) {
                    let del_keys = this.m_keys.splice(i, 1);
                    for (let k in del_keys) {
                        let data: any = del_keys[k];
                        this._data[data] = null;
                        this.size = this.m_keys.length;
                    }
                    return true;
                }
            }

            return false;
        }

        // /**
        //  * 获取某键对应的值
        //  * @param {String} key
        //  * @return {Object} value
        //  */
        get(key: any): V {
            for (let i = 0; i < this.m_keys.length; i++) {
                if (key === this.m_keys[i]) {
                    let keybuff: any = key;
                    return this._data[keybuff];
                }
            }
            return null;
        }
        getByIndex(index: number): V {
            for (let i = 0; i < this.m_keys.length; i++) {
                if (index ===  i) {
                    let keybuff: any = this.m_keys[i];
                    return this._data[keybuff];
                }
            }
            return null;
        }
        has(key: any): boolean {
            let ret = this.get(key);
            if (ret) {
                return true;
            }
            return false;
        }

        set(key: any, value: V) {
            this.delete(key); // 替换掉旧的数据
            this.m_keys.push(key);
            let keybuff: any = key;
            this._data[keybuff] = value;
            this.sort();
            this.size = this.m_keys.length;
            return true;
        }


        //排序
        sort() {
            // this.m_keys.sort( function (a:K, b:K) {
            //     return b < a;
            // });
        }


        //删除所有数据
        clear() {
            this.m_keys = null;
            this._data = null;
            this.m_keys = new Array();
            this._data = new Object();
            this.size = 0;
        }


        /**
         * 判断Map是否为空
         */
        isEmpty() {
            return this.m_keys.length == 0;
        }


    }


}
//# sourceMappingURL=MapGame.js.map