var game;
(function (game) {
    var MapGame = (function () {
        function MapGame() {
            this.keys = null;
            this.data = null;
            this.m_ponit = null;
            this.keys = new Array();
            this.data = new Object();
        }
        /**
         * 放入一个键值对
         * @param {String} key
         * @param {Object} value
         */
        MapGame.prototype.put = function (key, value) {
            this.remove(key); // 替换掉旧的数据
            this.keys.push(key);
            this.data[key] = value;
            this.sort();
            return true;
        };
        MapGame.prototype.set = function (key, value) {
            this.put(key, value);
        };
        //排序
        MapGame.prototype.sort = function () {
            //this.keys.sort();
            this.keys.sort(function (a, b) {
                return b < a;
            });
        };
        MapGame.prototype.begin = function () {
            var len = this.keys.length;
            for (var i = 0; i < len; i++) {
                var k = this.keys[i];
                return this.data[k];
            }
            return null;
        };
        MapGame.prototype.rbegin = function () {
            var len = this.keys.length;
            if (len == 0) {
                return null;
            }
            var k = this.keys[len - 1];
            return this.data[k];
        };
        MapGame.prototype.end = function () {
            return null;
        };
        MapGame.prototype.empty = function () {
            return this.keys.length == 0;
        };
        MapGame.prototype.pop = function () {
            var len = this.keys.length;
            for (var i = 0; i < len; i++) {
                var k = this.keys[i];
                this.remove(k);
                return true;
            }
        };
        MapGame.prototype.getnext = function (key) {
            for (var i = 0; i < this.keys.length; i++) {
                if (key === this.keys[i]) {
                    if (i + 1 < this.keys.length) {
                        var k = this.keys[i + 1];
                        var nodeinfo = this.data[k];
                        return nodeinfo;
                    }
                }
            }
            return null;
        };
        // /**
        //  * 获取某键对应的值
        //  * @param {String} key
        //  * @return {Object} value
        //  */
        // get(key) {
        //     return this.data[key];
        // }
        MapGame.prototype.getKey = function (index) {
            if (index == null) {
                return null;
            }
            return this.keys[index];
        };
        MapGame.prototype.getData = function (index) {
            if (index == null) {
                return null;
            }
            var k = this.keys[index];
            return this.data[k];
        };
        MapGame.prototype.get = function (key) {
            return this.getDataByKey(key);
        };
        MapGame.prototype.getDataByKey = function (key) {
            for (var i = 0; i < this.keys.length; i++) {
                if (key === this.keys[i]) {
                    return this.data[key];
                }
            }
            return null;
        };
        MapGame.prototype.hasData = function (key) {
            for (var i = 0; i < this.keys.length; i++) {
                if (key === this.keys[i]) {
                    return true;
                }
            }
            return false;
        };
        //删除所有数据
        MapGame.prototype.clear = function () {
            this.keys = null;
            this.data = null;
            this.keys = new Array();
            this.data = new Object();
        };
        /**
         * 删除一个键值对
         * @param {String} key
         */
        MapGame.prototype.remove = function (key) {
            for (var i = 0; i < this.keys.length; i++) {
                if (key === this.keys[i]) {
                    var del_keys = this.keys.splice(i, 1);
                    for (var k in del_keys) {
                        var data = del_keys[k];
                        this.data[data] = null;
                    }
                    return true;
                }
            }
            return false;
        };
        /**
         * 遍历Map,执行处理函数
         *
         * @param {Function} 回调函数 function(key,value,index){..}
         */
        MapGame.prototype.each = function (fn, point, data) {
            if (typeof fn != 'function') {
                return;
            }
            this.m_ponit = point;
            var len = this.keys.length;
            for (var i = 0; i < len; i++) {
                var k = this.keys[i];
                fn.call(this.m_ponit, k, this.data[k], data);
            }
        };
        /**
         * 获取键值数组
         * @return entity[{key,value},{key,value}]
         */
        MapGame.prototype.entrySet = function () {
            var len = this.keys.length;
            var entrys = new Array(len);
            for (var i = 0; i < len; i++) {
                entrys[i] = {
                    key: this.keys[i],
                    value: this.data[this.keys[i]]
                };
            }
            return entrys;
        };
        /**
         * 判断Map是否为空
         */
        MapGame.prototype.isEmpty = function () {
            return this.keys.length == 0;
        };
        /**
         * 获取键值对数量
         */
        MapGame.prototype.size = function () {
            return this.keys.length;
        };
        MapGame.prototype.getSize = function () {
            return this.keys.length;
        };
        MapGame.prototype.containsKey = function (key) {
            return this.keys.filter(function (v) {
                if (v === key) {
                    return key;
                }
            }).length > 0;
        };
        /**
         * 重写toString
         */
        MapGame.prototype.toString = function () {
            var s = "{";
            for (var i = 0; i < this.keys.length; i++) {
                var k = this.keys[i];
                s += k + "=" + this.data[k];
                if (this.keys.length > i + 1) {
                    s += ',';
                }
            }
            s += "}";
            return s;
        };
        /**
         * 解析字符串到Map
         * {a=A,b=B,c=B,}
         */
        MapGame.prototype.parserStringAndAddMap = function (str) {
            var count = 0;
            if (str && str.length > 0) {
                str = str.trim();
                var startIndex = str.indexOf("{"), endIndex = str.lastIndexOf("}");
                if (startIndex !== -1 && endIndex !== -1) {
                    str = str.substring(startIndex + 1, endIndex);
                    var arrs = str.split(",");
                    for (var i = 0; i < arrs.length; i++) {
                        var kv = arrs[i].trim();
                        if (kv.length > 0 && kv.indexOf("=") !== -1) {
                            var kv_arr = kv.split("=");
                            if (kv_arr.length == 2) {
                                if (this.put(kv_arr[0].trim(), kv_arr[1].trim())) {
                                    count++;
                                }
                                else {
                                    console.error('error: kv:' + kv);
                                }
                            }
                        }
                    }
                }
                else {
                    console.log("data error:" + str);
                }
            }
            else {
                console.log('data is not empty');
            }
            return count;
        };
        return MapGame;
    }());
    game.MapGame = MapGame;
})(game || (game = {}));
//# sourceMappingURL=MapGame.js.map 
//# sourceMappingURL=MapGame.js.map