module game {

    export class MapGame {
        keys: any = null;
        data: any = null;
        m_ponit: any = null;

        constructor() {
            this.keys = new Array();
            this.data = new Object();
        }

        /**
         * 放入一个键值对
         * @param {String} key
         * @param {Object} value
         */
        put(key, value) {
            this.remove(key); // 替换掉旧的数据
            this.keys.push(key);
            this.data[key] = value;
            this.sort();
            return true;
        }

        set(key, value) {
            this.put(key, value);
        }

        //排序
        sort() {
            //this.keys.sort();
            this.keys.sort(function (a, b) {
                return b < a;
            });
        }

        begin() {
            var len = this.keys.length;
            for (var i = 0; i < len; i++) {
                var k = this.keys[i];
                return this.data[k];
            }
            return null;
        }

        rbegin() {
            var len = this.keys.length;
            if (len == 0) {
                return null;
            }
            var k = this.keys[len - 1];
            return this.data[k];
        }

        end() {
            return null;
        }

        empty() {
            return this.keys.length == 0;
        }

        pop() {
            var len = this.keys.length;
            for (var i = 0; i < len; i++) {
                var k = this.keys[i];
                this.remove(k);
                return true;
            }
        }

        getnext(key) {
            for (var i = 0; i < this.keys.length; i++) {
                if (key === this.keys[i]) {
                    if (i + 1 < this.keys.length) {
                        var k = this.keys[i + 1];
                        var nodeinfo = this.data[k];
                        return nodeinfo;
                    }
                }
            }
            return null;
        }

        // /**
        //  * 获取某键对应的值
        //  * @param {String} key
        //  * @return {Object} value
        //  */
        // get(key) {
        //     return this.data[key];
        // }

        getKey(index) {
            if (index == null) {
                return null;
            }
            return this.keys[index];
        }

        getData(index) {
            if (index == null) {
                return null;
            }
            var k = this.keys[index];
            return this.data[k];
        }
        get(key):any {
          return  this.getDataByKey(key);
        }
        getDataByKey(key) {
            for (var i = 0; i < this.keys.length; i++) {
                if (key === this.keys[i]) {
                    return this.data[key];
                }
            }
            return null;
        }

        hasData(key) {
            for (var i = 0; i < this.keys.length; i++) {
                if (key === this.keys[i]) {
                    return true;
                }
            }
            return false;
        }

        //删除所有数据
        clear() {
            this.keys = null;
            this.data = null;
            this.keys = new Array();
            this.data = new Object();
        }

        /**
         * 删除一个键值对
         * @param {String} key
         */
        remove(key) {
            for (var i = 0; i < this.keys.length; i++) {
                if (key === this.keys[i]) {
                    var del_keys = this.keys.splice(i, 1);
                    for (var k in del_keys) {
                        var data = del_keys[k];
                        this.data[data] = null;
                    }
                    return true;
                }
            }
            return false;
        }

        /**
         * 遍历Map,执行处理函数
         *
         * @param {Function} 回调函数 function(key,value,index){..}
         */
        each(fn, point, data) {
            if (typeof fn != 'function') {
                return;
            }
            this.m_ponit = point;
            var len = this.keys.length;
            for (var i = 0; i < len; i++) {
                var k = this.keys[i];
                fn.call(this.m_ponit, k, this.data[k], data);
            }
        }

        /**
         * 获取键值数组
         * @return entity[{key,value},{key,value}]
         */
        entrySet() {
            var len = this.keys.length;
            var entrys = new Array(len);
            for (var i = 0; i < len; i++) {
                entrys[i] = {
                    key: this.keys[i],
                    value: this.data[this.keys[i]]
                };
            }
            return entrys;
        }

        /**
         * 判断Map是否为空
         */
        isEmpty() {
            return this.keys.length == 0;
        }

        /**
         * 获取键值对数量
         */
        size() {
            return this.keys.length;
        }

        getSize() {
            return this.keys.length;
        }

        containsKey(key) {
            return this.keys.filter(function (v) {
                    if (v === key) {
                        return key;
                    }
                }).length > 0;
        }

        /**
         * 重写toString
         */
        toString() {
            var s = "{";
            for (var i = 0; i < this.keys.length; i++) {
                var k = this.keys[i];
                s += k + "=" + this.data[k];
                if (this.keys.length > i + 1) {
                    s += ',';
                }
            }
            s += "}";
            return s;
        }

        /**
         * 解析字符串到Map
         * {a=A,b=B,c=B,}
         */
        parserStringAndAddMap(str) {
            var count = 0;
            if (str && str.length > 0) {
                str = str.trim();
                var startIndex = str.indexOf("{"), endIndex = str.lastIndexOf("}");
                if (startIndex !== -1 && endIndex !== -1) {
                    str = str.substring(startIndex + 1, endIndex);
                    var arrs = str.split(",");
                    for (var i = 0; i < arrs.length; i++) {
                        var kv = arrs[i].trim();
                        if (kv.length > 0 && kv.indexOf("=") !== -1) {
                            var kv_arr = kv.split("=");
                            if (kv_arr.length == 2) {
                                if (this.put(kv_arr[0].trim(), kv_arr[1].trim())) {
                                    count++;
                                }
                                else {
                                    console.error('error: kv:' + kv);
                                }
                            }
                        }
                    }
                }
                else {
                    console.log("data error:" + str);
                }
            }
            else {
                console.log('data is not empty');
            }
            return count;
        }
    }
}
//# sourceMappingURL=MapGame.js.map